#include <cli.h>

#include <IRremote.h>
#include <IRremoteInt.h>

/*
 * Send codes for cat feeder:
 *
 *  Time setting: FF827D
 *         Start: FFC23D
 *   Food amount: FFA25D
 *        Timing: FFE21D
 *          Plus: FFB24D
 *         Minus: FFF20D
 *        Record: FFAA55
 *          Play: FFEA15
 *        On/Off: FFDA25
 */
 
 IRsend irsend;
 
 Cli cli = Cli("Feeder> ", &Serial);
 
 const char *commandMap[] = { "settime", "start", "food", "timing",
                              "+", "-", "record", "play", "onoff" };
 const char *commandHelp[] = { "Set the time", "Dish out a serving",
                               "Set food amount", "Set timing",
                              "plus", "minus", "record", "play", "turn on or off" };
                              
const CommandProc commandProcMap[] = {
  &cmd_settime, &cmd_start, &cmd_food,
  &cmd_timing, &cmd_plus, &cmd_minus,
  &cmd_record, &cmd_play, &cmd_onoff };

const char commandMapSize = 9;

void cmd_settime() {
  irsend.sendNEC(0xFF827D, 32);
}

void cmd_start() {
  irsend.sendNEC(0xFFC23D, 32);
}

void cmd_food() {
  irsend.sendNEC(0xFFA25D, 32);
}

void cmd_timing() {
  irsend.sendNEC(0xFFE21D, 32);
}

void cmd_plus() {
  irsend.sendNEC(0xFFB24D, 32);
}

void cmd_minus() {
  irsend.sendNEC(0xFFF20D, 32);
}

void cmd_record() {
  irsend.sendNEC(0xFFAA55, 32);
}

void cmd_play() {
  Serial.println("Play");
  irsend.sendNEC(0xFFEA15, 32);
}

void cmd_onoff() {
  irsend.sendNEC(0xFFDA25, 32);
}

void setup() {
  Serial.begin(9600);
}

void loop() {
  if (Serial.available())
    cli.addChar(Serial.read());
}
