/* Cli library for Arduino */

#include "WProgram.h"
#include "cli.h"
#include <string.h>

Cli::Cli(char *_prompt, Print *pf) {
  _clear();
  prompt = _prompt;
  printfunc = pf;
}

// Clear the CLI buffer
void Cli::_clear() {
  _cliBufferLength = 0;
  for (int i = 0 ; i < BUFFER_LENGTH ; i++) {
    _cliBuffer[i] = '\0';
    if (i < COMMAND_LENGTH)
    _commandBuffer[i] = '\0';
  }
}

void Cli::addChar(char c) {
  if (c == 0xD || c == ';') {
    printfunc->print("\r\n");
    _parseAndRun();
    return;
  }
  if (c == 0x8) {
    if (_cliBufferLength > 0) {
      _cliBuffer[--_cliBufferLength] = '\0';
      printfunc->print("\010 \010");
    }
    return;
  }
  _cliBuffer[_cliBufferLength++] = c;
  if (_cliBufferLength >= BUFFER_LENGTH)
    _cliBufferLength = BUFFER_LENGTH-1;
  else
    printfunc->write(c);
}

char Cli::_strncmp(const char *s1, const char *s2, char n) {
const char *idx1 = s1;
const char *idx2 = s2;
char numindex = 0;

  while (idx1 != 0 && idx2 != 0 && numindex < n) {
    if (*idx1 != *idx2)
      return 0;
    idx1++; idx2++; numindex++;
  }
  return 1;
}

char Cli::_strlen(const char *s) {
char len = 0;
  for ( ; *s != '\0' ; s++)
    len++;
  return len;
}

void Cli::_strclr(char *s, char len) {
  for (char i = 0 ; i < len ; i++) {
    *s = '\0';
    *s++;
  }
}

char *Cli::getArg() {
char *ptr;
  ptr = &_cliBuffer[0];
  while (*ptr != '\0' && *ptr != ' ')
    *ptr++;
  if (*ptr == ' ')
    *ptr++;
  return ptr;
}

char *Cli::getArg(char n) {
char *ptr;

  ptr = &_cliBuffer[0];
  for (char i = 0 ; i < _cliBufferLength ; i++) {
    if (n == 0) break;
    if (*ptr == '\0') break;
    if (*ptr == ' ') n--;
    *ptr++;
  }
  _strclr(&_argBuffer[0], COMMAND_LENGTH);
  for (char i = 0 ; i < COMMAND_LENGTH ; i++) {
    _argBuffer[i] = *ptr++;
    if (*ptr == '\0' || *ptr == ' ') break;
  }
  return &_argBuffer[0];
}

void Cli::_parseAndRun() {
char i;
  if (_cliBufferLength == 0) {
    // Blank line
    printfunc->print(prompt);
    return;
  }
  if (_cliBuffer[0] == '?' && _cliBuffer[1] == '\0') {
    printfunc->println("Command help:");
    for (char i = 0 ; i < commandMapSize ; i++) {
      printfunc->write(' ');
      printfunc->print(commandMap[i]);
      printfunc->print("  ");
      printfunc->println(commandHelp[i]);
    }
    printfunc->print(prompt);
    _clear();
    return;
  }

  for (i = 0 ; i < _cliBufferLength ; i++)
    if (_cliBuffer[i] != '\0' && _cliBuffer[i] != ' ')
      _commandBuffer[i] = _cliBuffer[i];
    else
      break;
  for (i = 0 ; i < commandMapSize ; i++) {
    if (_strlen(&_commandBuffer[0]) == _strlen(commandMap[i]) &&
        _strncmp(&_commandBuffer[0], commandMap[i], _strlen(commandMap[i]))) {
      // Found our command
      commandProcMap[i]();
      _clear();
      printfunc->print(prompt);
      return;
    }
  }
  _clear();
  printfunc->println("Unknown command. '?' for help.");
  printfunc->print(prompt);
}
